% function load_xBTLg
% Function load_xBTLg is a Matlab script that processes files in the
% WOCE hydrographic bottle exchange file format and returns a structure 
% array of the data contained in the fields.
% Usage:  BTL = load_xBTL (FileList)
%   where  FileList  is a cell array of (path) file identifiers.
%                    e.g.  FileList = {'file1.csv'  'file2.csv'}
%
% Version 1.3          Aug 2006          R. Goldsmith
% Version 1.4          Dec 2007          R. Goldsmith
%         1.4  load_XBTL/load_XBTL3 becomes load_XBTLg, more general.
%              Modified to make SAMPNO,BTLNBR strings
%              The original version cleverly used csvread with the
%              assumption that everything was numbers after the first
%              two columes. That pretty much swallowed the whole rest
%              of the file in one fell swoop. But now we are essentially
%              reduced to processing line by line. Which is what I did
%              originally. Cleverness usually doesn't pay,
% Caveats: Okay, it's not exactly the exchange format as furnished
%   through the data centers. They have some ugly characters embedded
%   in the data fields (besides the expected EXPOCODE and SECT_ID)
%   that have been replaced by blanks. Notably offensive were the
%   BOTTLE_NUMBER (BTLNBR), and some codes.
%   Also discovered that the organization creating the Bottle transport
%   .cdv exchange format files put more than one section identifiers
%   in a file. This royally torqued the method I used to speed up the
%   processing, namely to read the first one and build the rest the same.
%   For now we've just parsed up the files to enforce the seemingly
%   innocuous rule that the file identifier reflects the section
%   identifier.

function  BTL = load_xBTL (FileList)


flog = 1;
mFields = 0;
     % To force these to be cell initially for operations but
     % eventually want it to be strings. Probably to pre-allocate to 
     % some large number but then would have to restrict output later.
FieldList = {};
UnitsList = {};
len6 = 6;
len7 = 7;
len14 = 14;
len15 = 15;
theEXPOCODE = '';

     % Initialize structure.
BTL = struct ([]);
is = 0;          % The number of bottles in the structure.

     % Check for FileList.
if (nargin < 1)
  disp ('  *** load_xBTL ERROR: input arguments required.');
  help  load_xBTL
  return
  end  % end input argument check

     % Check for type of argument.
if (ischar (FileList) )
  theList = {FileList};
 elseif (iscell (FileList) )
  theList = FileList;
 else
  disp ('  *** load_xBTL ERROR: input argument must be a cell of');
  disp ('                       file identifier strings.');
  help  load_xBTL
  return
 end  % argument type check.


% tic
nFiles = length (theList);
iFile = 1;
     % Here is the start of the file loop.
     % Sorry it is not indented.
while (iFile <= nFiles)
  theFile = theList{iFile};

     % Read the records.
fbtl = fopen (theFile, 'r');
if (fbtl == -1)
  disp (['  *** load_xBTL ERROR:  opening file: '  theFile] );
 break 
  end  % end of file open error
fprintf (flog, '  ... processing file: %s\n', theFile);

NR = 1;
lHeader = 1;

while 1
  theLine = fgetl (fbtl);
  if (~ischar (theLine) )
    break
   elseif (strncmp (theLine, 'END_DATA', 8) == 1)
    lHeader = 1;
    break
   end
          % Display first line.
  if (NR == 1)
    % disp (theLine);
    fprintf (flog, '      %s\n', theLine);
    end

          % Check for Field labels and units.
  if (strncmp (theLine, 'EXPOCODE', 8) == 1)
    lHeader = 0;
    theEXPOCODE = 'EXPOCODE';
    nBot = 0;          % The number of bottles in the file.
    % disp (theLine);
    theFields = textscan (theLine, '%s', 'delimiter', ',');
          % Replace any dashes as they are bad in structure fields.
    theFields = strrep (theFields{1}, '-', '_');
    theFields = strrep (theFields, '+', '_');
    theFields = lower (theFields);

    theLine = fgetl (fbtl);
    % disp (theLine);
    theUnits = textscan (theLine, '%s', 'delimiter', ',');
    theUnits = lower (theUnits{:} );
          % Check the length of theUnits. Sometimes if the last units
          % are empty the last comma is followed by the EOL, which
          % is not considered a field terminator. This results in
          % theUnits length being one less than theFields length.
          % Put in a patch, not foolproof but should be pretty good.
    if (length (theUnits) == (length (theFields)-1) )
      theUnits(length(theFields)) = {''};
      disp ('  *** load_xBTL WARNING: adding empty field to UNITS list.');
      end  % if patched theUnits.

    nFields = length (theFields );
    % disp ('Using fields:')
    for i = 1:nFields
      % disp ([theFields{i}  '('  theUnits{i}  ')'] )
      Stem = sprintf ('%-15.15s', theFields{i} );
      if (isempty (strmatch ({Stem}, FieldList, 'exact') ))
        mFields = mFields + 1;
        FieldList(mFields) = {Stem};
        Stem = sprintf ('%-15.15s', theUnits{i} );
        UnitsList(mFields) = {Stem};
        end  % check if field is already in list.
      end  % head list
    continue;
    end  %  if we are processing EXPOCODE (fields and units list) header.

          % Flush any headers.
  if (lHeader == 1)
    NR = NR + 1;
    continue
    end


          % Do something with the data.
          % The first two fields are ASCII, the rest numeric.
          % Now the 5th and 6th as also strings.
  % keyboard
  if (~isempty (theEXPOCODE) )
    inx = findstr (theLine, ',');
    theValS = textscan (theLine(1:length(theLine)), '%s', 'delimiter', ',');
    nBot = nBot + 1;
    is = is + 1;
          % Is this the first bottle in the file.
    stem = sprintf ('%-14.14s', theValS{1}{1} );
    if (nBot == 1)
      theEXPOCODE = stem;
      end
          % If this is the first bottle create the structure.
    if (is == 1)
      BTL = struct (theFields{1}, theEXPOCODE );
      end  % set initial element.

          % We want to force the string length, 
          %   14 expocode
          %    6 sect_id
          %    7 sampno, btlnbr
          % Validate consistent EXPOCODE.
    if (strcmp (stem, theEXPOCODE) == 0)
      display (['  *** load_XBTLg: EXPOCODE mismatch after bottle record ' ...
               num2str(nBot) ]);
      break
      end  % if expocode mismatch.

          % We are doing this to preserve the blanks in the fields.
    BTL = setfield (BTL, theFields{1}, {is,1:len14}, stem );
    stem = sprintf ('%-6.6s', theValS{1}{2} );
    BTL = setfield (BTL, theFields{2}, {is,1:len6}, stem );
    stem = sprintf ('%-7.7s', theValS{1}{5} );
    BTL = setfield (BTL, theFields{5}, {is,1:len7}, stem );
    stem = sprintf ('%-7.7s', theValS{1}{6} );
    BTL = setfield (BTL, theFields{6}, {is,1:len7}, stem );

          % Definitly not the Matlab way of doing things.
    for i = [3,4,7:nFields]
      atem = sscanf (char(theValS{1}(i)), '%f');
          % Check for -999 values. It would be nice to do this
          % at the read but a) I'm not sure that the process wouldn't
          % gag on the text string nan where it's expecting a numeric
          % and b) there are several variations, ie.e -999, -999.0, -999.000
          % etc.
          % I should also mention that this does not account for
          % subsequent zero filling when a new variable is introduced.
          % or a previously defined field is not in the current file.
          % Not sure if better to do here or at the very end.
      if (atem == -999)
        atem = nan;
        end  % if there are missing values.
      BTL = setfield (BTL, theFields{i}, {is}, atem);
      end  %  loading numeric values.

    NR = NR + 1;
    % if (NR > 10)  break, end
    end  % processing a data record.

  end  % while reading records.

fclose (fbtl);

fprintf (flog, '      number of bottles: %i\n', nBot);


iFile = iFile + 1;
end  % while processing files.


     % Add in the field  names and units lists.
     % Sort them to match ordered fields later.
[FieldList, ixtem] = sort (FieldList);
UnitsList = UnitsList(ixtem);
BTL = setfield (BTL, 'fields', {1:mFields,1:len15}, char(FieldList) ); 
BTL = setfield (BTL, 'units', {1:mFields,1:len15}, char(UnitsList) ); 
     % Sort the fields so when they look at the structure is is nice.
BTL = orderfields (BTL);

% toc

