%  Stokes1;  Ekmanlayer w/o latitiude
%  Ekmanlayer;  an elementary upper ocean Ekman layer
%  Solve a 1-D diffusion flow numerically.  This
%  version has rotation, and thus makes an Ekman 
%  layer.  The IC is a state of rest.  The fluid is forced
%  by an imposed stress at the top of the column (z=0)
%  The lower boundary condition can be made free-slip to minimize
%  the effect of finite depth, or noslip. 
%

clear
close all
format compact

%  display the text string str2
str2(1) =  {'Stokes1' };
str2(2) =  {'  '};
str2(3) =  {'An upper ocean layer driven by a constant '};
str2(4) =  {'northward stress of 0.1 Pa.  Solved by an elementary'};
str2(5) =  {'numerical method.  You will be asked to enter:'};
str2(6) =  {'the water depth, meters, nominal = 50' }; 
str2(7) =  {'the diffusivity, cm^2 sec^{-1}, nominal = 100'};
str2(8) =  {'the number of days to integrate, nominal = 10 '}; 
str2(9) =  {''};
str2(10) = {''};
str2(11) = {'Jim Price, November, 2005.'};

hf3 = figure(1);
clf reset
% set(hf3,'Position',[50 200 550 550])
set(gca,'Visible','off')
text(0, 0.50, str2,'FontSize',13,'HorizontalAlignment','left');

L = input('Enter the water depth, m (20 - 100 - 500)   ');
dz = 5.;          %  the grid interval, m
z = 0:-dz:-L ;    %  the grid defined

Kcgs = input('Enter the diffusivity, cm^2 sec^-1 (20 - 100 - 1000)  ');
K = Kcgs/1.e4;

ndays = input('Enter the days to integrate, (3 - 10 - 30)  ');
dt = 0.4*dz^2/K; %  the time step, set this so that 
b = dt*K/dz^2;    %  b is less then 0.5 for numerical stability
nstep = round(ndays*8.64e4/dt);

%  specify the latitude 
%  lat = input('Enter the latitude, degrees, (0 - 30 -  90)  ');
lat = 0.;
f = 2*7.29e-5*sin(lat*pi/180);
IP = (2*pi/f)/8.64e4;

rho = 1025.;     %  nominal constant density of water
%  the (northward) stress
tauBC = 0.1/rho; %  imposed constant stress/density as upper BC
Ektrans = tauBC/f;
Ekdepth = sqrt(2*K/f);

u = 0*z;         %  the dependent variable, a velocity
v = u;
[nn nz] = size(u);
uavg = zeros(1,nz);
vavg = uavg; 
navg = 0;

figure(1)
clf reset
title('upper ocean Ekman layer, \tau_y = 0.1 Pa')

set(0,'DefaultLineLineWidth',1.4)
set(0,'DefaultTextFontSize',14)
set(0,'DefaultAxesLineWidth',1.3)
set(0,'DefaultAxesFontSize',14)

%  begin time-stepping

 for j=1:nstep
       
 delsqu = [0 diff(u,2) 0];    
 delsqv = [0 diff(v,2) 0];

u = u + b*delsqu + dt*f*v;
v = v + b*delsqv - dt*f*u;

% apply the surface BC next
    u(1) = u(2);
    v(1) = v(2)  + dz*tauBC/K;
% the bottom BC (free-slip)
    u(nz) = u(nz-1);
    v(nz) = v(nz-1);
    
% or no-slip
   u(nz) = 0;
   v(nz) = 0;
 
%  compute and store some diagnostics

time(j) = (j-1)*dt;

%  sum the currents to make a time-average
   uavg = uavg + u;
   vavg = vavg + v;
   navg = navg + 1;   

%  time series of surface current and the transport
usurf(j) = u(1);
vsurf(j) = v(1);
mid = round(nz/3) -1;
umid(j) = u(mid);
vmid(j) = v(mid);
mid2 = 2*mid;
udeep(j) = u(mid2);
vdeep(j) = v(mid2);

transu(j) = (sum(u) - u(1))*dz;
transv(j) = (sum(v) - v(1))*dz;

topstress(j) = -K*(u(2) - u(1))/dz;
botstress(j) = -K*(u(nz) - u(nz-1))/dz;

nv1 = 2;
nv2 = nz-1;

topwork(j) = - (u(nv1)*K*(u(nv1+1) - u(nv1-1))/(2*dz) +...
  v(nv1)*K*(v(nv1+1) - v(nv1-1))/(2*dz))  ;
botwork(j) =  u(nv2)*K*(u(nv2+1) - u(nv2-1))/(2*dz) + ...
   v(nv2)*K*(v(nv2+1) - v(nv2-1))/(2*dz);

ds = 0.;
kes = 0.;
for m=nv1:nv2
   shru = (u(m+1) - u(m))/dz;
   shrv = (v(m+1) - v(m))/dz;
   ds = ds - dz*K*(shru^2 + shrv^2);
   kes = kes + 0.5*dz*(u(m)^2 + v(m)^2);
end
diss(j) = ds;
ke(j) = kes;

efold = 1 - exp(-1);
ssum = cumsum((sqrt(u.*u + v.*v)+1.e-4));
ze(j) = -interp1(ssum,z,efold*ssum(nz), 'linear');
zK(j) = sqrt(0.5*K*time(j));


%  vorticity budget over a volume from nv1 to nv2

avgvort(j) = u(nv1) - u(nv2);
topvflux(j) = -K*delsqu(nv1)/dz^2;
botvflux(j) = -K*delsqu(nv2)/dz^2;
netvflux(j) = botvflux(j) - topvflux(j);
tippingx(j) = f*(u(nv1) - u(nv2));

avgvorty(j) = v(nv1) - v(nv2);
topvfluxy(j) = -K*delsqv(nv1)/dz^2;
botvfluxy(j) = -K*delsqv(nv2)/dz^2;
netvfluxy(j) = botvfluxy(j) - topvfluxy(j);
tippingy(j) = f*(v(nv1) - v(nv2)); 

%  plot profiles every nplot steps
nplot = round(1.e4/dt);
if mod(j,nplot) == nplot - 1
   hold on
    
figure(1)
clf reset

 x3 = ones(2,nz); 
 zero = 0.*x3;
 y3 = x3;   
 z3 = x3; 

 for m=1:nz;
 x3(1,m) = 0.;
 x3(2,m) = u(m);
 y3(1,m) = 0.;
 y3(2,m) = v(m);
 z3(1,m) = z(m);
 z3(2,m) = z(m); 
 end;

 hp =  plot3(x3,y3,z3,'b-'); 
 set(hp,'LineWidth',1.3)
 xlabel('east speed, m/s')
 ylabel('north speed, m/s')
 zlabel('depth, m') 
 
 %  show the wind stress direction
 clear x3 y3 z3;
 x3 = [0 0]; y3 = [0 0.1]; z3 = [0 0]; 
 hold on
 hp =  plot3(x3, y3, z3, 'r-' );
 set(hp,'LineWidth',1.5)
 plot3([0 0], [0 0], [0 -L], 'k')
 tday = time(j)/8.64e4;
 text(0.4, 0.3, -20., [num2str(tday, 3), ' days'])
 grid
 axis([-0.5 0.5 0 1 -L 0])
 drawnow
 pause(0.1)
 
end   % if on whether to plot this time step

end   %  loop on j for time step


%  compute and plot diagnostics

%  time-integrate the top and bottom stress to compare with the
%   transport
totstr = cumsum(topstress-botstress)*dt;
%botstr = cumsum(botstress)*dt;

tday = time/8.64e4;
figure(4)
clf reset
subplot(2,1,1)
plot(tday, ze, tday, zK,':')
xlabel('time, days')
ylabel('bl thickness, m')
legend('e-folding', 'sqrt(0.5*K*t)')
legend boxoff
title('boundary layer thickness')

figure(15)
clf reset
tday = time/8.64e4;
subplot(2,1,1)
plot(tday, usurf, tday, umid, tday, udeep)
ylabel('east, m/s')
grid
title('east and north currents, z = 0, L/3 2L/3')
subplot(2,1,2)
plot(tday, vsurf, tday, vmid, tday, vdeep)
ylabel('north, m/s')
xlabel('time, days')
grid


figure(11)
clf reset
plot(transu, transv)
hold on
plot([0 Ektrans], [0 0],'r+')
Tx = mean(transu);
Ty = mean(transv);
plot([0 Tx], [0 Ty], 'g')
xlabel('U trans, m^2 sec^{-1} ')
ylabel('V trans, m^2 sec^{-1} ')
% axis([-0.1 0.15 -0.25 0.])
axis('equal')
title('transport with time, time-average and Ekman transport')


figure(13)
clf reset

uavg = uavg/navg;
vavg = vavg/navg;

 x3 = ones(2,nz); 
 zero = 0.*x3;
 y3 = x3;   
 z3 = x3; 

 for m=1:nz;
 x3(1,m) = 0.;
 x3(2,m) = uavg(m);
 y3(1,m) = 0.;
 y3(2,m) = vavg(m);
 z3(1,m) = z(m);
 z3(2,m) = z(m); 
 end;

hp =  plot3(x3,y3,z3,'b-'); 
set(hp,'LineWidth',1.3)
 xlabel('east speed, m/s')
 ylabel('north speed, m/s')
 zlabel('depth, m') 
 
 %  show the wind stress direction
 clear x3 y3 z3;
 x3 = [0 0 0]; y3 = [0 0 0.1]; z3 = [-L 0 0]; 
 hold on
 hp =  plot3(x3, y3, z3, 'r-' );
 set(hp,'LineWidth',1.5)
 grid
 title('time-average current and stress')
 

dothis = 0;
if dothis == 1

figure(12)
clf reset

uavg = uavg/navg;
vavg = vavg/navg;

plot(uavg, z, vavg, z)
xlabel('east and north currents, m/sec')
ylabel('depth, m')
title('Mean current profiles')

figure(14)
clf reset
% plot(uavg, vavg, '*')
hold on
plot([0 0], [0 00.06], 'r')

for j=1:nz
   uh = [0 uavg(j)];
   vh = [0 vavg(j)];
   hold on 
   plot(uh, vh, 'LineWidth', 1.3)
end
xlabel('east speed, m/s')
ylabel('north speed, m/s')
title('Hodograph, dz = 2.5 m')

%  energy budget
figure(6)
clf reset

%subplot(2,1,1)
dkedt = diff(ke)/dt;
tdkedt = cumsum(diff(time));
plot(time, topwork,':', time, botwork,'-.', ...
    time, diss, '--', tdkedt, dkedt)
legend('top work rate', 'bot work rate', 'dissipation rate','dKE/dt')
ylabel('Work rate, m*m/s*s*s')
xlabel('time, s')
title('energy budget, 20-30 m')
% axis([0 4e6 -5e-5 5e-5])

%  vorticity budget
figure(7)
clf reset

subplot(2,1,1)
dvortdt = diff(avgvort)/dt;
timedv = cumsum(diff(time));


plot(time, -netvflux, '-', timedv, dvortdt, '-.', time, tippingx, '--')
%  note, the sign on net flux is negative ????

ylabel('vorticity fluxes,x')
legend('-div(flux)', 'dvort/dt','tipping')
title('Overall vorticity budget')
axis([0 6e5 -1.e-5 1.e-5])

subplot(2,1,2)
dvortdty = diff(avgvorty)/dt;
timedv = cumsum(diff(time));
plot(time, netvfluxy, '-',  ...
   timedv, dvortdty, '-.', time, tippingy, '--')
xlabel('time, s')
ylabel('vorticity fluxes, y')

legend('-div(flux)','dvort/dt', 'tipping')
axis([0 6e5 -1.e-5 1.e-5])
end   %  if on dothis

