%  THCboxmodel
%  
%  A variant of Stommel's (wonderful) 1961 model of convection in coupled   
%  boxes.  This is set up to look somewhat more like a marginal sea
%  connected to an open and presumably infinite ocean.  The marginal sea  
%  has a temperature, y, and a salinity, x. The density difference between 
%  ocean and the sea the is given by d = -y + R*x, where we are assuming 
%  R = beta/alpha > 1, typically.  There is an exchange (or flushing) 
%  between the ocean and the marginal sea that has a rate 
%  dr*lambda, where dr is the density difference between the boxes, and
%  notice that the flushing does not depend upon the sign of the density 
%  anomal (vertical stratification is not recognized).  
%  
%  Written by Jim Price, October, 2005.  Modified in Aug 2007
%  Public domain for all educational purposes.
%

set(0,'DefaultLineLineWidth',1.6)
set(0,'DefaultTextFontSize',14)
set(0,'DefaultAxesLineWidth',1.6)
set(0,'DefaultAxesFontSize',14)
 
clear; format compact; 
nn = 0;


%  specify the ocean (reservoir) T and S, held fixed at:
yres = 0;
xres = 0;

%  the atmospheric temperature, also fixed
ya = -1.;

%  the thermal and haline expansion coefficients;  the sign 
%  of alpha is applied in the eqn of state
alpha = 1.;
beta = 2.;

EMP = -0.01;     %  evap minus precip over the marginal sea, < 0 is precip
lambda = 0.2;    %  the factor multiplying dr to calculate the exchange
Cq = 0.1;        %  the heat flux coefficient with the atmosphere

%  the time step (non-d) and the number of steps
dtau = 0.05;    %  the time step in non-d time; 0.05 seems fine
nstep = 5000;   %  number of time steps; 5000 seems to be enough  
                %  to insure convergence to a steady state
%                 
%  This first part of the code is set up to do experiments over a range
%  of initial T,S or y,x from -1 to 1.  The increment of T and S 
%  are set by delT and delS = 1/ni,  where ni is the number of
%  experiments (n1 = 4 to 20 is reasonable).

ni = 5;
delT = 1/ni;   
delS = delT;

for n1=-1:delT:1
for n2=-1:delS:1
      
if n1==-1 | n1==1 | n2==-1 | n2==1  %  skip all of the initial points that do not
                                  %   fall on the boundary   
x(1) = n1;   %  set the initial sea temperature
y(1) = n2;   %  set the initial sea salinity

for m = 2:nstep  
    
tau(m) = m*dtau;  %  the non-d time

%  calculate the density difference wrt the (fixed) ocean 
dr(m) = beta*(x(m-1) - xres) - alpha*(y(m-1) - yres);   

%  calculate the exchange between the sea and the ocean
q = abs(dr(m))*lambda;                        

%  step ahead the temperature;  notice the Euler forward 'method'
y(m) = y(m-1) + dtau*Cq*(ya - y(m-1)) + dtau*abs(q)*(yres - y(m-1));  

%  step ahead the salinity
x(m) = x(m-1) + dtau*EMP + dtau*abs(q)*(xres - x(m-1)); 

end  %  end of time step loop

if nn == 0;
nn = 1;   
%  make a time plot of the first case only 
figure(1)
clf reset

subplot(2,1,1)
plot(tau, y, 'g', tau, x, 'b')
legend('temperature', 'salinity')
legend boxoff
ylabel('temp and salinity')
title('Experiment 1,1')

subplot(2,1,2)
plot(tau, dr)
xlabel('time, non-d')
ylabel('density diff')
  
%  contour the density (proportional to the flushing rate), and add 
%  the (T,S) trajectories on top of the contours
   
for k1 = 1:11
for k2 = 1:11  
    xm(k1) = -1 + 0.1*(k1 - 1);
    ym(k2) = -1 + 0.1*(k2 - 1);
    dm(k2,k1) = (beta*xm(k1)  - alpha*ym(k2));
end
end

figure(2)
clf reset
dc = -3:0.5:3;
c = contour(xm, ym, dm, dc,'k');
clabel(c);
xlabel('salinity ')
ylabel('temperature')
hold on
end   %  the if on nn = 0 

[m1 m2] = size(x);

%  plot the individual trajectories. 
%  color code according to which equilibrium point
%  the trajectory ends up on. this will likely have be 
%  reset if the model parameters  are changed.
   
if dr(m2) >= 0
plot(x, y, 'r')
plot(x(m2), y(m2), '*r', 'linewidth', 1.1)
else 
plot(x, y, 'c')
plot(x(m2), y(m2), '*c', 'linewidth', 1.1)
end
axis([-1 0 -1 0])
title('trajectories for various starting T, S')

end  %  the if to gate out non-boundary values of n1, n2
end  %  loop on n1
end  %  loop on n2
text(-0.7, -0.9, 'cold state')
text(-0.25, -0.7, 'warm state')

%  end of the first calculation.................


%  now, try varying the salinity or temperature abruptly.........

for k=1:10

clear x y tau dr1 t1 x1 y1 dr1
%  initialize in the 'cold' steady state;  these are for the original
%    model parameters only
x(1) = -0.46;
y(1) = -0.83;
x1(1) = x(1);
y1(1) = y(1);
t1(1) = 0.;

disp(' Starting from the cold state, impose a salinity and temperature')
disp(' jump at t = 0, and see what happens.......')
sjump = input('  enter the salinity jump (>9 = quit)  ');
if sjump >= 8 break
else
tjump = input('  enter the temperature jump  ');

for m = 2:4000   
tau(m) = m*dtau;  %  the non-d time  
dr(m) = beta*(x(m-1) - xres) - alpha*(y(m-1) - yres);   %  the density anomaly
q = abs(dr(m))*lambda;                        %  the flushing rate
y(m) = y(m-1) + dtau*Cq*(ya - y(m-1))...  %  step ahead temperature
   + dtau*abs(q)*(yres - y(m-1));      
x(m) = x(m-1) + dtau*EMP ...  %  step ahead salinity 
  + dtau*abs(q)*(xres - x(m-1)); 

%  impose the salinity or temperature jump here 
if m == 1000
    x(m) = x(m) + sjump;
    y(m) = y(m) + tjump;
end

%  save data for plotting
x1(m) = x(m);
y1(m) = y(m);
dr1(m) = dr(m);
t1(m) = tau(m);
end  %  end of time step loop

figure(3)
clf reset
hold on
plot(t1-50, x1, t1-50, y1)
xlabel(' time')
ylabel(' temp (green) and salinity (blue)')
title('response to T or S change when in cold state')
legend('salinity', 'temperature')
axis([-50 200 -1 0])
drawnow
figure(3)

end    %  loop on k   
end    %  if on sjump
