
%  mixandadvect_1d:  Mixing and advection in one dimension.  
%  Advect a tracer with a specified, uniform upward velocity, W,
%  (in effect, a pipe or channel flow) while diffusing with a 
%  specified K.  The BCs are specified end points. 
%  
%  This is a very simple problem, but useful to help 
%  appreciate how the Peclet number sets the tracer profile
%  in the steady state. 
%
%  by Jim Price, Oct 2005.  modified Oct 2006, Aug 2007.
%
%  public domain for all personal, educational purposes
%

clear all, close all

%  the seawater subdirectory has to be in the path, somehow
%  in this case, seawater is a subdirectory at the same level as this 
%    script (you may have to do something quite different)
path(path,'..\seawater') 

%  also needs the mid-Paciifc data file K2_1.mat taken from Knauss's Table
%           2.1

set(0,'DefaultAxesFontSize',13)
set(0,'DefaultTextFontSize',13)
set(0,'DefaultAxesLineWidth', 1.3)
set(0,'DefaultLineLineWidth', 1.0)
format compact

%  set the ocean depth, m
L = 5.e3;

%  Eta is the mean upwelling rate, m/year;   5 is nominal
%  Eta = 5.
%  the values shown in parens are (low to nominal to large)

Eta = input('  enter the upwelling rate, meters per year (1 to 5 to 20) ')
W = Eta/(365.*8.64e4);   %  the global mean upwelling velocity

%  K = 1.e-4;               %  Munk's inferred K
%  K = 0.2e-4;              %  smaller, more like TRE size
K = input('  enter a diffusivity, m^2 sec^-1 (1.e-5 to 1.e-4 to 1.e-3) ') 
disp('  for this Eta and K, the Peclet number WL/K is  ')
Pe = W*L/K     %  the Peclet number

%  define the grid 
dz = 0.02*L;      %  the grid interval
z = -L:dz:0;      %  the spatial coordinate
nz = length(z);

% set the time step and the number of steps to take
dtdays = 20.;            %  time step in days
dt = dtdays*8.64e4;      %  the time step is in seconds 
nstep = 50000;           %  do this many steps (could be made adjustable)
years = dt*nstep/(365.*8.64e4);
advectL = dt*nstep*W/L   %  the normalized advection distance, should exceed 1

%  check that the diffusive time step limit is OK
disp('  the non-d K should be < 1/4;  Knd is ') 
Knd = K/(dz^2/dt)    %  the dimensional diffusion coefficient;  
                     %  should be < 1/4 for numerical stability

%  set the initial condition for a scalar tracer variable a
a = ones(1,nz);
a(z>-0.5*L) = 20.0;
asurf = 20.;

%  arrays of indices needed to estimate derivatives
ip = [1 3:nz nz];
im = [1 1:nz-2 nz];
i0 = [1 2:nz-1 nz];

figure(1); clf reset;  % send the intreim plots here
xlabel('potential temperature, C')
ylabel('depth, m')
title(['Peclet = WL/K = ', num2str(Pe,2)])
axis([0 20 -6000 0])

% enough of this chit chat,  let's start diffusing and advecting!
for n = 1:nstep

%  estimate the Laplacian required for diffusion
d2a = ((a(ip) + a(im)) - 2*a)/dz^2;

%  estimate the first derivative by an upwind difference
%    needed for advection
da = (a(i0) - a(im))/dz;

%  now define the BCs: zeroes on the ends of d2a amounts
%  to leaving the end points fixed in time at their initial values 
d2a(1) = 0.;
d2a(nz) = 0.; 
da(1) = 0.;
da(nz) = 0.;

% now take a forward time step
 a = a + dt*(K*d2a - W*da);

plotint = 1000;     %  plot the a array every this many time steps
dyears = floor(plotint*dtdays/365);

if mod(n,plotint) == 1
hold on
if n == 1 
    text(10., -4500., ['plotted every ', num2str(dyears), ' years'])
end
plot(a, z, 'linewidth', 1.2)
if n == plotint+1; xlabel('tracer'); ylabel('depth');  grid on; end
end                %  if on plotint or not  
drawnow

end   % the end of the time step loop on n

% add the last time step as a heavy green line
hold on
plot(a/asurf, z/L, 'g', 'linewidth', 1.6)

%  replot the last time solution along with Knauss Table 2.1 data 
figure(2)
clf reset
plot(a, z, 'g', 'linewidth', 1.6)

%  add the T(z) from Knauss's Table 2_1 mid-Pacific Temp profile
%  note that the file K2_1.mat will have to be in this directory or path

load K2_1
lat = A.lat;
Z = A.pre;
T = A.tem;
S = A.sal;
% change depth, Z, to pressure
Z = sw_pres(Z, lat);
%  potential temperature
theta = sw_ptmp(S, T, Z, 0.);

hold on
plot(theta, -Z, 'r', 'linewidth', 1.5)
grid on
axis([0 20 -6000 0])
legend('numerical solution', 'mid-Pacific T(z)')
legend boxoff
xlabel('potential temperature, C')
ylabel('depth, m')
title(['Peclet = WL/K = ', num2str(Pe,2)])

%  all done with the numerical solution

%  the code below is for some diagnostics and checks
%  it works if you have symbolic math - otherwise, just skip
   
%  now compute the steady solution by symbolic math to see if
%  this incredibly simple-minded numerical solution actually
%  solves the equations we intended (in the steady state)

disp('  now try some symbolic math - don`t worry if this fails...')    
syms as zs
as = dsolve('Das = (K/W)*D2as', 'as(-L) = 1', 'as(0) = 20', 'zs');
disp('  the symbolic math steady solution:')
pretty(simplify(factor(as)))
das = diff(as);      %  the z derivative of the solution
d2as = diff(das);    %  the second derivative

%  evaluate the symbolic math solution
for j=1:nz
zs = -L + (j-1)*dz;
zr(j) = zs;
ar(j) = subs(as);
dar(j) = subs(das);
d2ar(j) = subs(d2as);
ls(j) = (max(a) - min(a))/dar(j)/L;
advflux(j) = W*ar(j);
diffflux(j) = -K*dar(j);
Konlyflux(j) = -K*(a(nz)-a(1))/L;
end
%  plot the symbolic math result to compare with the numerical solution
figure(1)
hold on
plot(ar, zr, 'r.')
% axis([0 1 -1 0])
title(['Peclet = WL/K = ', num2str(Pe,2)])
axis([0 20 -5500 0])

skipthis = 1;
if skipthis == 0

Ko = Konlyflux(10);
figure(4)
clf reset
plot(advflux/Ko, zr)
hold on
plot(diffflux/Ko, zr, 'r')
plot(Konlyflux/Ko, zr, 'g')
netflux = advflux + diffflux;
plot(netflux/Ko, zr, 'k')
legend('advec', 'diffus', 'Konly', 'sum')
title(' normalized fluxes')



%  evaluate the local length scale da/dz
%  how does this compare with the Pe-inferred value?
clear dar
for j=2:nz-1
    dar(j) = (ar(j+1) - ar(j-1))/(2*dz);
end
dar(1) = 1.;   dar(nz) = 1.0; 
lscale = (1/L)*(max(ar) - min(ar))*dar.^-1;
q = -1./(exp(-Pe) -1)

figure (4)
clf reset
plot(lscale, zr, 'b.')
hold on
plot(ls, zr, 'r')
% axis([0 Pe -1 0])
xlabel('internal length scale, \delta a/(L*(da/dz))')
ylabel('depth')
title(['Pe = WL/K = ', num2str(Pe,2)])
grid on

end    % the end on skipthis

