% partslope; geostrophic adjustment of a particle on a slope
%
% compute the motion of a single particle on a 
% rotating earth, numerically. This is useful for seeing how an 
% Ekman balance or a geostrophic balance appears as the time average 
% of a start up problem, or, as the instantaneous balance if there 
% is some friction and you can wait a bit. May also be useful as a 
% first look at some forms of numerical time integration of ODEs.
%
% By Jim Price, October, 1995; modified in October 2005, Aug 2007
% Public domain for educational purposes.
%
% The equations of motion for a particle on a rotating earth are:
%      du/dt - f*v = Fx/rho - k*u,    for the east component, and
%      dv/dt + f*u = Fy/rho - k*v,    for the north component.
% 
% d()/dt is an ordinary time derivative since this model is meant
%   to describe the motion of a single particle (and not a fluid!), 
% u and v are the east and north velocity components of the particle,
% Fx and Fy are the east and north forces per unit volume applied
%   to the particle (in a fluid problem these could be boundary stress),
% rho is the nominal density of sea water,
% f is the usual Coriolis parameter, and
% k is a linear decay coefficient that represents, say, bottom drag.

clear all
close all
% set default graphics things
set(0,'DefaultTextFontSize',14)
set(0,'DefaultAxesFontSize',14)
set(0,'DefaultAxesLineWidth',1.2)
set(0,'DefaultLineLineWidth',1.6)


%  display the text string str2
str2(1) =  {'partslope-b:' };
str2(2) =  {'' };
str2(3) =  {'Solve for the motion of a particle sitting on a '};
str2(4) =  {'sloping constant pressure surface and subject to the '};
str2(5) =  {'Coriolis force and to a linear drag.  This '}; 
str2(6) =  {'demsonstrates a local version of geostrophic adjustment,  '}; 
str2(7) =  {'and the effect of friction on a geostrophic motion. '}; 
str2(8) =  {'  '}; 
str2(9) =  {'You may enter a new:'}
str2(10) = {'deltaz, the change in depth per 100 km, m, 0.2 - 3'};
str2(11) =  {'latitude,  degrees, -90 to +90, '}; 
str2(12) =  {'e-folding time for linear drag, days, 0 -to 9999, '};
str2(13) =  {'initial alongslope speed of the particle, Ui:'};
str2(14) =  {'   ufrac = Ui/Ugeo = 0 to start from rest,  '};
str2(15) =  {'           = 1 to start in geostrophic balance, and'};
str2(16) =  {'           > 1 for a super geostrophic balance.'};

hf3 = figure(9);
clf
set(hf3,'Position',[50 50 560 600])
set(gca,'Visible','off')
text(-.05, 0.70, str2,'FontSize',12,'HorizontalAlignment','left')

deltaz = 1.
hdeltaz = uicontrol('Style', 'Edit', 'Units', 'Normalized', ...
	'Position',  [0.05 0.26 0.35 0.06], ...
	'FontSize', 10, ...
	'Callback', 'eval(get(hdeltaz,''string''))', ...
	'String', 'deltaz = 1. ');

latitude = 40.
hlat = uicontrol('Style', 'Edit', 'Units', 'Normalized', ...
	'Position',  [0.05 0.18 0.35 0.06], ...
	'FontSize', 10, ...
	'Callback', 'eval(get(hlat,''string''))', ...
	'String', 'latitude = 40 ');

efold = 10.;
hefold = uicontrol('Style', 'Edit', 'Units', 'Normalized', ...
	'Position',  [0.05 0.10 0.35 0.06], ...
	'Callback', 'eval(get(hefold,''string''))', ...
	'FontSize', 10, ...
	'String', 'efold  = 10 ');

ufrac = 0.;
hufrac = uicontrol('Style', 'Edit', 'Units', 'Normalized', ...
	'Position',  [0.05 0.02 0.35 0.06], ...
	'Callback', 'eval(get(hufrac,''string''))', ...
	'FontSize', 10, ...
	'String', 'ufrac = 0');

hgo = uicontrol('Style', 'Pushbutton', 'Units', 'Normalized', ...
	'Position',  [0.80 0.10 0.1 0.06], ...
	'Callback', 'uiresume(9)', ...
	'FontSize', 10, ...
	'String', 'Start');

hquit = uicontrol('Style', 'Pushbutton', 'Units', 'Normalized', ...
	'Position',  [0.80 0.02 0.1 0.06], ...
	'Callback', 'lquit = 1; uiresume(9)', ...
	'FontSize', 10, ...
	'String', 'Quit');
lquit = 0;

while (lquit == 0)
disp  ('  ... waiting for control panel action.')
uiwait(9)
if lquit == 1; break; end;

day = 8.64e4;

% lat = input('Enter the latitude (deg)');
f = 2.*7.292e-5*sin(latitude*pi/180.);

if abs(f) <= 1.e-10; f = 1.e-8; end   % don't let f = 0

% The force (Fx, Fy) is evaluated as if it were a buoyancy force. 
% As now set up, the force is northward only i.e., forcex = 0 and 
% forcey > 0.

slope = deltaz/(100*1000);   %  slope, deltah per 100 km
g = 9.8;
delrho = 1025.;
rho = 1025.;         % nominal sea water density.
h = 100.;            %  column thickness

% There is a linear drag or frictional force, that will cause an 
% e-folding of an unforced current in time efold (days).

% efold = input('Enter the e-folding time for drag (days, 1 - 9999999)' );

k = 1./(efold*day);           % compute the linear drag consistent with this efold.

spd = 100.;                   % time steps per day
ndays = 10.;                  % the number of days to run (10 is plenty).
dt = 1800.;                   % the time step, sec. 
nstep = ndays*day/dt;         % the number of time steps needed to go ndays.

u = 1:nstep; v=1:nstep; t=1:nstep;  % set up some vectors to store the solution. 

% the (geostrophic) scales used to plot things
%  these are not used in partslope_a to simplify things a little
usc = g*(delrho/rho)*slope/f;
xsc = 0.001*usc/f;
tsc = day;
zsc = 1000.*xsc*slope;
esc = h*usc^2;


%  use instead these 'dimensional' scales
  tsc = day;
  xsc = 1000.;
  ysc = 1000.;
  zsc = 1.;
  
u(1) = ufrac*usc; v(1) = 0; t(1) = 0.;      % define the initial condition. 
fricwork(1) = 0.;
%  the initial position of the particle
x(1) = 0;
y(1) = 0.;   

% Begin a loop that will step the equations ahead in time.

for i=1:nstep-1

t(i+1) = t(i) + dt;      % increment the time.

Fx = 0.;
Fy = g*delrho*slope;

% A fourth order Runga-Kutta method; Euler forward would have been fine.

du1 =  dt*( Fx/rho + f*v(i) - k*u(i) );   
dv1 =  dt*( Fy/rho - (f*u(i) + k*v(i)) ); 

du2 =  dt*( Fx/rho + f*(v(i)+dv1/2.)  - k*(u(i)+du1/2.) );
dv2 =  dt*( Fy/rho - (f*(u(i)+du1/2.) + k*(v(i)+dv1/2.)) ); 

du3 =  dt*( Fx/rho + f*(v(i)+dv2/2.)  - k*(u(i)+du2/2.) );
dv3 =  dt*( Fy/rho - (f*(u(i)+du2/2.) + k*(v(i)+dv2/2.)) );

du4 =  dt*( Fx/rho + f*(v(i)+dv3)  - k*(u(i)+du3) );
dv4 =  dt*( Fy/rho - (f*(u(i)+du3) + k*(v(i)+dv3)) );

u(i+1) = u(i) + du1/6. + du2/3. + du3/3. + du4/6.;
v(i+1) = v(i) + dv1/6. + dv2/3. + dv3/3. + dv4/6.;

forx(1,i+1) = Fx/rho;
forx(2,i+1) = f*v(i);
forx(3,i+1) = -k*u(i);

fory(1,i+1) = Fy/rho;
fory(2,i+1) =  -f*u(i);
fory(3,i+1) = -k*v(i);

x(i+1) = x(i) + dt*u(i+1);   
y(i+1) = y(i) + dt*v(i+1);
z(i+1) = y(i)*slope;

ke(i+1) = 0.5*h*(u(i+1)^2 + v(i+1)^2);
pe(i+1) = -g*delrho*h*(y(i+1)*slope);
fricwork(i+1) = fricwork(i) - k*h*(u(i+1)^2 + v(i+1)^2)*dt;
esum(i+1) = ke(i+1) +  pe(i+1) - fricwork(i+1);

end                  % the end of the time stepping loop.

%  make a movie-like plot of position and depth

x3 = -2000:500:2000;
y3 = -50:50:200;
x3 = x3*1000.;
y3 = y3*1000.,

nx3 = max(size(x3));
ny3 = max(size(y3));

for i=1:nx3
for j=1:ny3
    z3(j,i) = -y3(j)*slope;
end
end

notthis = 1;
if notthis == 0
figure(4)
clf reset
hsurf4 = surfl (x3, y3, z3);
set (hsurf4, 'FaceColor', [.8 .9 1.]);
depth = -y*slope;
hold on
hp9 = plot3(x/xsc, y/xsc, depth/zsc, 'b');
set(hp9, 'Linewidth', 1.6)
view(-70, 60);
xlabel(' x/(g`\alpha/f^2)')
ylabel(' y/(g`\alpha/f^2)')
zlabel(' z/(g`\alpha^2/f^2)')
axis([0 60 0 4 -4 0])
title('particle on a slope w/ rotation and friction')
end   %  if on notthis


m = 0;
for j=1:10:nstep
  m = m + 1;

  if m == 1
    hf1 =  figure(1)
    clf reset
    set(hf1,'Position',[300 170 700 600])
    hsurf1 = surfl(x3/xsc, y3/xsc, z3);
    set (hsurf1, 'FaceColor', [.8 .9 1.]);
    depth = -y*slope;
    hold on
    hp9 = plot3 (x(1)/xsc, y(1)/xsc, depth(1)/zsc, 'b*');
    set(hp9, 'erasemode', 'none')
    % Do not give initial line plot.
    % plot3 (x/xsc, y/xsc, (depth+50)/zsc, 'm-');

    view(-30, 70);
%    xlabel(' x/(g`\alpha/f^2)')
%    ylabel(' y/(g`\alpha/f^2)')
%    zlabel(' z/(g`\alpha^2/f^2)')
xlabel('x distance, km')
ylabel('y distance, km')
zlabel('z depth, m')
 %   axis([0 60 0 4 -4 0])
    title('particle on a slope with rotation and friction')
    
tip = num2str(t(j)/tsc);
tipstr = [tip, ' days'];
hth = text(1500., 150., tipstr);

    drawnow
    end

 if m > 1
    hold on
    plot3 (x(jl:j)/xsc, y(jl:j)/xsc, depth(jl:j)/zsc, 'b-', ...
          'erasemode', 'none', 'LineWidth', 1.6);
    pause(0.05)   %  slow things down a bit
    if mod(m,2) == 1
      tip = num2str(t(j)/tsc);
      tipstr = ['time, days = ', tip];
      set(hth, 'String', tipstr)
    end
    drawnow
 end   %  if to plot or not

  jl = j;
  end  %  loop on j for plotting each time step

figure(2)
clf reset

subplot(2,1,1)
plot(t/tsc,u,t/tsc,v)
grid
legend('along','across')
legend boxoff
xlabel('time, days')
ylabel('U, m/sec')
title('along and across slope current components')

subplot(2,1,2)
plot(x/xsc, y/xsc)
grid
axis('square')
xlabel('along slope distance, x, km')
ylabel('across slope, y, km')
axis('equal')

skip9 = 1;
if skip9 == 0
figure(3)
clf reset
subplot(2,1,1)

pe1 = pe(2);
pe = pe - pe1;
esum1 = esum(2);
esum = esum - esum1;

plot(t/tsc, ke/esc, t/tsc, pe/esc, t/tsc, fricwork/esc, t/tsc, esum/esc)
xlabel('time/(2\pi/f)')
ylabel('energy/(g`\alpha/f)^2')
legend('kinetic E', 'potential E', 'frictional W', 'E + W')
legend boxoff
title('work and energy changes')
hold on
plot(t/tsc, 0*t, 'linewidth', 0.5)
end     %  if on skip9

format compact
% Check the transport and a few other things.
% Assumes that the external force is in the north direction only.

f                         %  Coriolis parameter
k                         %  the linear drag coefficient
ekman = k/f               %  their ratio, Ekman number
angtoisobaths = max(y)/max(x)       %  approx angle of trajectory to isobaths
alpha = atan(ekman);      % 

eky = -Fx/(rho*f);        %  component-wise Ekman numbers 
ekx = Fy/(rho*f);
ufric = Fy/( rho*(f*cos(alpha) + k*sin(alpha)) );  
urat = ufric/ekx;
vfric = ufric*sin(alpha);

% the end used to stop the script with the quit button 
end  
